@@ -6,7 +6,6 @@ import json
 
 from autots.tools.shaping import long_to_wide
 import random
-from autots.tools.profile import data_profile
 from autots.tools.shaping import subset_series
 from autots.tools.shaping import simple_train_test_split
 from autots.evaluator.auto_model import TemplateEvalObject
@@ -34,7 +33,6 @@ class AutoTS(object):
         constraint (float): when not None, use this value * data st dev above max or below min for constraining forecast values. Applied to point forecast only, not upper/lower forecasts.
         ensemble (str): None, 'simple', 'distance'
         initial_template (str): 'Random' - randomly generates starting template, 'General' uses template included in package, 'General+Random' - both of previous. Also can be overriden with self.import_template()
-        figures (bool): Not yet implemented
         random_seed (int): random seed allows (slightly) more consistent results.
         holiday_country (str): passed through to Holidays package for some models.
         subset (int): maximum number of series to evaluate at once. Useful to speed evaluation when many series are input.
@@ -48,6 +46,7 @@ class AutoTS(object):
         num_validations (int): number of cross validations to perform. 0 for just train/test on final split.
         models_to_validate (int): top n models to pass through to cross validation. Or float in 0 to 1 as % of tried.
             0.99 is forced to 100% validation. 1 evaluates just 1 model.
+            If horizontal or probabilistic ensemble, then additional min per_series models above the number here may be added to validation.
         max_per_model_class (int): of the models_to_validate what is the maximum to pass from any one model class/family.
         validation_method (str): 'even', 'backwards', or 'seasonal n' where n is an integer of seasonal
             'backwards' is better for recency and for shorter training sets
@@ -74,9 +73,8 @@ class AutoTS(object):
                  max_generations: int = 5,
                  no_negatives: bool = False,
                  constraint: float = None,
-                 ensemble: str = None,
+                 ensemble: str = 'simple',
                  initial_template: str = 'General+Random',
-                 figures: bool = False,
                  random_seed: int = 2020,
                  holiday_country: str = 'US',
                  subset: int = None,
@@ -94,7 +92,7 @@ class AutoTS(object):
                  drop_data_older_than_periods: int = 100000,
                  model_list: str = 'default',
                  num_validations: int = 2,
-                 models_to_validate: float = 0.05,
+                 models_to_validate: float = 0.15,
                  max_per_model_class: int = None,
                  validation_method: str = 'even',
                  min_allowed_train_percent: float = 0.5,
@@ -127,7 +125,7 @@ class AutoTS(object):
         self.model_interrupt = model_interrupt
         self.verbose = int(verbose)
         if self.ensemble == 'all':
-            self.ensemble = 'simple,distance'
+            self.ensemble = 'simple,distance,horizontal-max,probabilistic-max'
 
         if self.forecast_length == 1:
             if metric_weighting['contour_weighting'] > 0:
@@ -143,18 +141,16 @@ class AutoTS(object):
                                'AverageValueNaive', 'GLS', 'SeasonalNaive',
                                'GLM', 'ETS', 'ARIMA', 'FBProphet',
                                'RollingRegression', 'GluonTS',
-                               'UnobservedComponents', 'VARMAX',
-                               'VECM', 'DynamicFactor', 'MotifSimulation',
-                               'WindowRegression']
+                               'UnobservedComponents', 'VAR',
+                               'VECM', 'WindowRegression']
         if model_list == 'superfast':
             self.model_list = ['ZeroesNaive', 'LastValueNaive',
                                'AverageValueNaive', 'GLS', 'SeasonalNaive']
         if model_list == 'fast':
             self.model_list = ['ZeroesNaive', 'LastValueNaive',
                                'AverageValueNaive', 'GLS', 'GLM', 'ETS',
-                               'RollingRegression', 'WindowRegression',
-                               'GluonTS', 'VAR',
-                               'SeasonalNaive', 'UnobservedComponents',
+                               'WindowRegression', 'GluonTS',
+                               'VAR', 'SeasonalNaive',
                                'VECM', 'ComponentAnalysis']
         if model_list == 'probabilistic':
             self.model_list = ['ARIMA', 'GluonTS', 'FBProphet',
@@ -176,20 +172,22 @@ class AutoTS(object):
                                'ComponentAnalysis']
 
         # generate template to begin with
-        if initial_template.lower() == 'random':
-            self.initial_template = RandomTemplate(50,
-                                                   model_list=self.model_list)
-        elif initial_template.lower() == 'general':
+        initial_template = str(initial_template).lower()
+        if initial_template == 'random':
+            self.initial_template = RandomTemplate(
+                50, model_list=self.model_list)
+        elif initial_template == 'general':
             from autots.templates.general import general_template
             self.initial_template = general_template
-        elif initial_template.lower() == 'general+random':
+        elif initial_template == 'general+random':
             from autots.templates.general import general_template
             random_template = RandomTemplate(40, model_list=self.model_list)
-            self.initial_template = pd.concat([general_template,
-                                               random_template],
-                                              axis=0).drop_duplicates()
+            self.initial_template = pd.concat(
+                [general_template, random_template], axis=0).drop_duplicates()
+        elif isinstance(initial_template, pd.DataFrame):
+            self.initial_template = initial_template
         else:
-            print("Input initial_template either unrecognized or not yet implemented. Using Random.")
+            print("Input initial_template unrecognized. Using Random.")
             self.initial_template = RandomTemplate(50)
 
         # remove models not in given model list
@@ -218,13 +216,14 @@ class AutoTS(object):
                 return "Initiated AutoTS object"
 
     def fit(self, df,
-            date_col: str = 'datetime', value_col: str = 'value',
+            date_col: str = None, value_col: str = None,
             id_col: str = None, future_regressor=[],
-            weights: dict = {}, result_file: str = None):
+            weights: dict = {}, result_file: str = None,
+            grouping_ids=None):
         """Train algorithm given data supplied.
 
         Args:
-            df (pandas.DataFrame): Datetime Indexed
+            df (pandas.DataFrame): Datetime Indexed dataframe of series, or dataframe of three columns as below.
             date_col (str): name of datetime column
             value_col (str): name of column containing the data of series.
             id_col (str): name of column identifying different series.
@@ -233,11 +232,13 @@ class AutoTS(object):
             result_file (str): results saved on each new generation. Does not include validation rounds.
                 ".csv" save model results table.
                 ".pickle" saves full object, including ensemble information.
+            grouping_ids (dict): currently a one-level dict containing series_id:group_id mapping.
         """
         self.weights = weights
         self.date_col = date_col
         self.value_col = value_col
         self.id_col = id_col
+        self.grouping_ids = grouping_ids
 
         # convert class variables to local variables (makes testing easier)
         forecast_length = self.forecast_length
@@ -276,17 +277,21 @@ class AutoTS(object):
         np.random.seed(random_seed)
 
         # convert data to wide format
-        df_wide = long_to_wide(
-            df, date_col=self.date_col,
-            value_col=self.value_col,
-            id_col=self.id_col,
-            frequency=self.frequency,
-            na_tolerance=self.na_tolerance,
-            drop_data_older_than_periods=self.drop_data_older_than_periods,
-            aggfunc=self.aggfunc,
-            drop_most_recent=self.drop_most_recent,
-            verbose=self.verbose
-            )
+        if date_col is None and value_col is None:
+            df_wide = pd.DataFrame(df)
+            assert type(df.index) is pd.DatetimeIndex, "df index is not pd.DatetimeIndex"
+        else:
+            df_wide = long_to_wide(
+                df, date_col=self.date_col,
+                value_col=self.value_col,
+                id_col=self.id_col,
+                frequency=self.frequency,
+                na_tolerance=self.na_tolerance,
+                drop_data_older_than_periods=self.drop_data_older_than_periods,
+                aggfunc=self.aggfunc,
+                drop_most_recent=self.drop_most_recent,
+                verbose=self.verbose
+                )
 
         # clean up series weighting input
         if not weighted:
@@ -397,6 +402,7 @@ class AutoTS(object):
             template_cols=template_cols,
             random_seed=random_seed,
             model_interrupt=self.model_interrupt,
+            grouping_ids=self.grouping_ids,
             verbose=verbose)
         model_count = template_result.model_count
 
@@ -444,6 +450,7 @@ class AutoTS(object):
                 startTimeStamps=self.startTimeStamps,
                 template_cols=template_cols,
                 model_interrupt=self.model_interrupt,
+                grouping_ids=self.grouping_ids,
                 random_seed=random_seed, verbose=verbose
                 )
             model_count = template_result.model_count
@@ -480,6 +487,7 @@ class AutoTS(object):
                     startTimeStamps=self.startTimeStamps,
                     template_cols=template_cols,
                     model_interrupt=self.model_interrupt,
+                    grouping_ids=self.grouping_ids,
                     random_seed=random_seed, verbose=verbose)
                 model_count = template_result.model_count
                 # capture results from lower-level template run
@@ -536,6 +544,21 @@ class AutoTS(object):
         if str(self.max_per_model_class).isdigit():
             validation_template = validation_template.sort_values('Score', ascending=True, na_position='last').groupby('Model').head(self.max_per_model_class).reset_index(drop=True)
         validation_template = validation_template.sort_values('Score', ascending=True, na_position='last').head(self.models_to_validate)
+        # add on best per_series models (which may not be in the top scoring)
+        ensy = ['horizontal', 'probabilistic']
+        if any(x in ensemble for x in ensy) and not self.subset_flag and self.models_to_validate > 30:
+            model_results = self.initial_results.model_results
+            mods = pd.DataFrame()
+            if 'horizontal' in ensemble:
+                mods = pd.concat(
+                    [mods, self.initial_results.per_series_mae.idxmin()])
+            if 'probabilistic' in ensemble:
+                mods = pd.concat(
+                    [mods, self.initial_results.per_series_spl.idxmin()])
+            per_series_val = model_results[model_results['ID'].isin(mods.iloc[:,0].unique().tolist())]
+            validation_template = pd.concat([validation_template, per_series_val], axis=0)
+            validation_template = validation_template.drop_duplicates(
+                subset=['Model', 'ModelParameters', 'TransformationParameters'])
         validation_template = validation_template[self.template_cols]
 
         # run validations
@@ -616,6 +639,7 @@ class AutoTS(object):
                     startTimeStamps=self.startTimeStamps,
                     template_cols=self.template_cols,
                     model_interrupt=self.model_interrupt,
+                    grouping_ids=self.grouping_ids,
                     random_seed=random_seed, verbose=verbose,
                     validation_round=(y + 1))
                 model_count = template_result.model_count
@@ -656,12 +680,12 @@ or otherwise increase models available."""
                                                     ens_templates],
                                                    axis=0)
                 if 'hdist' in ensemble:
-                    per_series = self.initial_results.per_series_mae1.copy()
+                    per_series = self.initial_results.per_series_rmse1.copy()
                     temp = per_series.mean(axis=1).groupby(level=0).count()
                     temp = temp[temp >= (num_validations + 1)]
                     per_series = per_series[per_series.index.isin(temp.index)]
                     per_series = per_series.groupby(level=0).mean()
-                    per_series2 = self.initial_results.per_series_mae2.copy()
+                    per_series2 = self.initial_results.per_series_rmse2.copy()
                     temp = per_series2.mean(axis=1).groupby(level=0).count()
                     temp = temp[temp >= (num_validations + 1)]
                     per_series2 = per_series2[per_series2.index.isin(temp.index)]
@@ -716,6 +740,7 @@ or otherwise increase models available."""
                     startTimeStamps=self.startTimeStamps,
                     template_cols=template_cols,
                     model_interrupt=self.model_interrupt,
+                    grouping_ids=self.grouping_ids,
                     random_seed=random_seed, verbose=verbose)
                 # capture results from lower-level template run
                 template_result.model_results['TotalRuntime'].fillna(
@@ -806,6 +831,7 @@ or otherwise increase models available."""
         return out
 
     def predict(self, forecast_length: int = "self",
+                prediction_interval: float = 'self',
                 future_regressor=[], hierarchy=None,
                 just_point_forecast: bool = False,
                 verbose: int = 'self'):
@@ -813,6 +839,9 @@ or otherwise increase models available."""
 
         Args:
             forecast_length (int): Number of periods of data to forecast ahead
+            prediction_interval (float): interval of upper/lower forecasts.
+                defaults to 'self' ie the interval specified in __init__()
+                if prediction_interval is a list, then returns a dict of forecast objects.
             future_regressor (numpy.Array): additional regressor, not used
             hierarchy: Not yet implemented
             just_point_forecast (bool): If True, return a pandas.DataFrame of just point forecasts
@@ -824,6 +853,8 @@ or otherwise increase models available."""
         verbose = self.verbose if verbose == 'self' else verbose
         if forecast_length == 'self':
             forecast_length = self.forecast_length
+        if prediction_interval == 'self':
+            prediction_interval = self.prediction_interval
 
         # if the models don't need the regressor, ignore it...
         if not self.used_regressor_check:
@@ -834,33 +865,61 @@ or otherwise increase models available."""
             self.future_regressor_train = self.future_regressor_train.reindex(
                 index=self.df_wide_numeric.index)
 
-        df_forecast = PredictWitch(
-            self.best_model,
-            df_train=self.df_wide_numeric,
-            forecast_length=forecast_length,
-            frequency=self.frequency,
-            prediction_interval=self.prediction_interval,
-            no_negatives=self.no_negatives,
-            constraint=self.constraint,
-            future_regressor_train=self.future_regressor_train,
-            future_regressor_forecast=future_regressor,
-            holiday_country=self.holiday_country,
-            startTimeStamps=self.startTimeStamps,
-            random_seed=self.random_seed, verbose=verbose,
-            template_cols=self.template_cols)
-
-        trans = self.categorical_transformer
-        df_forecast.forecast = trans.inverse_transform(
-            df_forecast.forecast)
-        df_forecast.lower_forecast = trans.inverse_transform(
-            df_forecast.lower_forecast)
-        df_forecast.upper_forecast = trans.inverse_transform(
-            df_forecast.upper_forecast)
-
-        if just_point_forecast:
-            return df_forecast.forecast
+        # allow multiple prediction intervals
+        if isinstance(prediction_interval, list):
+            forecast_objects = {}
+            for interval in prediction_interval:
+                df_forecast = PredictWitch(
+                    self.best_model,
+                    df_train=self.df_wide_numeric,
+                    forecast_length=forecast_length,
+                    frequency=self.frequency,
+                    prediction_interval=interval,
+                    no_negatives=self.no_negatives,
+                    constraint=self.constraint,
+                    future_regressor_train=self.future_regressor_train,
+                    future_regressor_forecast=future_regressor,
+                    holiday_country=self.holiday_country,
+                    startTimeStamps=self.startTimeStamps,
+                    grouping_ids=self.grouping_ids,
+                    random_seed=self.random_seed, verbose=verbose,
+                    template_cols=self.template_cols)
+                trans = self.categorical_transformer
+                df_forecast.forecast = trans.inverse_transform(
+                    df_forecast.forecast)
+                df_forecast.lower_forecast = trans.inverse_transform(
+                    df_forecast.lower_forecast)
+                df_forecast.upper_forecast = trans.inverse_transform(
+                    df_forecast.upper_forecast)
+                forecast_objects[interval] = df_forecast
+            return forecast_objects
         else:
-            return df_forecast
+            df_forecast = PredictWitch(
+                self.best_model,
+                df_train=self.df_wide_numeric,
+                forecast_length=forecast_length,
+                frequency=self.frequency,
+                prediction_interval=prediction_interval,
+                no_negatives=self.no_negatives,
+                constraint=self.constraint,
+                future_regressor_train=self.future_regressor_train,
+                future_regressor_forecast=future_regressor,
+                holiday_country=self.holiday_country,
+                startTimeStamps=self.startTimeStamps,
+                grouping_ids=self.grouping_ids,
+                random_seed=self.random_seed, verbose=verbose,
+                template_cols=self.template_cols)
+            trans = self.categorical_transformer
+            df_forecast.forecast = trans.inverse_transform(
+                df_forecast.forecast)
+            df_forecast.lower_forecast = trans.inverse_transform(
+                df_forecast.lower_forecast)
+            df_forecast.upper_forecast = trans.inverse_transform(
+                df_forecast.upper_forecast)
+            if just_point_forecast:
+                return df_forecast.forecast
+            else:
+                return df_forecast
 
     def results(self, result_set: str = 'initial'):
         """Convenience function to return tested models table.
@@ -897,10 +956,15 @@ or otherwise increase models available."""
             if any(x in self.ensemble for x in ens_list):
                 temp = self.initial_results.model_results
                 temp = temp[temp['Ensemble'] >= 2]
+                temp = temp[temp['Exceptions'].isna()]
                 export_template = export_template.merge(
                     temp, how='outer',
                     on=export_template.columns.intersection(
                         temp.columns).to_list())
+                export_template['Score'] = generate_score(
+                    export_template,
+                    metric_weighting=self.metric_weighting,
+                    prediction_interval=self.prediction_interval)
             if str(max_per_model_class).isdigit():
                 export_template = export_template.sort_values(
                     'Score', ascending=True
@@ -1011,24 +1075,39 @@ or otherwise increase models available."""
 
 
 class AutoTSIntervals(object):
-    """Autots looped to test multiple prediction intervals."""
+    """Autots looped to test multiple prediction intervals. Experimental.
+    
+    Runs max_generations on first prediction interval, then validates on remainder.
+    Most args are passed through to AutoTS().
 
-    def fit(self, prediction_intervals, import_template, forecast_length,
+    Args:
+        interval_models_to_validate (int): number of models to validate on each prediction interval.
+        import_results (str): results from run on same data to load, `filename.pickle`.
+            Currently result_file and import only save/load initial run, no validations.
+    """
+
+    def fit(self, prediction_intervals, 
+            forecast_length,
             df_long, max_generations, num_validations,
             validation_method, models_to_validate,
             interval_models_to_validate,
             date_col, value_col, id_col=None,
+            import_template=None,
+            import_method='only',
+            import_results=None,
+            result_file=None,
             model_list='all',
             metric_weighting: dict = {
                 'smape_weighting': 1,
-                'mae_weighting': 1,
-                'rmse_weighting': 0,
+                'mae_weighting': 0,
+                'rmse_weighting': 1,
                 'containment_weighting': 0,
                 'runtime_weighting': 0,
                 'spl_weighting': 10,
                 'contour_weighting': 0
                 },
             weights: dict = {},
+            grouping_ids=None,
             future_regressor=[],
             model_interrupt: bool = False,
             constraint=2, no_negatives=False,
@@ -1056,10 +1135,15 @@ class AutoTSIntervals(object):
                 model_interrupt=model_interrupt,
                 models_to_validate=models_to_validate)
             if import_template is not None:
-                current_model.import_template(import_template, method='only')
+                current_model = current_model.import_template(
+                    import_template, method=import_method)
+            if import_results is not None:
+                current_model = current_model.import_results(import_results)
             current_model = current_model.fit(
                 df_long, future_regressor=future_regressor,
                 weights=weights,
+                grouping_ids=grouping_ids,
+                result_file=result_file,
                 date_col=date_col, value_col=value_col,
                 id_col=id_col)
             current_model.initial_results.model_results['interval'] = interval
@@ -1068,6 +1152,8 @@ class AutoTSIntervals(object):
             temp = current_model.initial_results.per_series_spl
             per_series_spl = pd.concat([per_series_spl, temp], axis=0)
             if runs == 0:
+                result_file = None
+                import_results = None
                 import_template = current_model.export_template(
                     None, models='best', n=interval_models_to_validate)
             runs += 1
@@ -1126,6 +1212,7 @@ class AutoTSIntervals(object):
                 future_regressor_forecast=future_regressor,
                 holiday_country=self.holiday_country,
                 startTimeStamps=self.startTimeStamps,
+                grouping_ids=self.grouping_ids,
                 random_seed=self.random_seed, verbose=verbose,
                 template_cols=self.template_cols)
 
@@ -1140,7 +1227,6 @@ class AutoTSIntervals(object):
         return forecast_objects
 
 
-
 def fake_regressor(df_long, forecast_length: int = 14,
                    date_col: str = 'datetime', value_col: str = 'value',
                    id_col: str = 'series_id',
